/**********************************************************/
/*** Analysis of results of diluted ferromagnet         ***/
/***                                                    ***/
/*** Version 0.1.0.0                                    ***/
/***                                                    ***/
/*** A.K. Hartmann May 2005                             ***/
/***                                                    ***/
/**********************************************************/

#include <stdio.h>
#include <math.h>
#include <string.h>

#include "makros_ah.h"

/* function commentary header (empty): */

/*******************  *****************/
/**  **/
/** PARAMETERS: (*)= return-paramter         **/
/**  **/
/** RETURNS:                               **/
/**  **/
/******************************************/


#include <stdio.h>
#include "llstat.h"

#define MAXLINE 1000
#define ENERGY 1
#define MAG 2
#define MAG2 3

int main(int argc, char **argv)
{
  int t, argz = 1;
  FILE *file;
  char line[MAXLINE];             /* buffer for one line of input file */
  char filename[MAXLINE];
  int num_scanned;                        /* number of parameters read */
  llstat_t *stat, *stat2;  /* histos where data is stored for analysis */
  llstat_t *stat_e;     /* histogram where data is stored for analysis */
  int do_correl;                    /* calculate correlation function? */
  double correl, correl_norm;              /* autocorrelation function */
  double sum1, sum2, sum3;             /* for autocorrelation function */
  int t_min;                                 /* ignore dae for t<t_min */
  int t_max;                           /* for autocorrelation function */
  int time, t_last, delta_time;                        /* data read in */
  double  energy, mag, mag2;                           /* data read in */
  double *value;                      /* for measuring autocorrelation */
  int max_data;                                     /* size of value[] */
  double mean, var;                 /* for normalizing autocorrelation */
  int c_value;          /* measure autocorrelation for ENERGY or MAG ? */
  double binder, binder_err;      /* Binder parameter + error estimate */
  double binder2, binder_err2;      /* Binder parameter + error estimate */
  
  t_max = 10; max_data = 1000; c_value = MAG; do_correl = 0;

  while((argz<argc)&&(argv[argz][0] == '-'))              /* treat options */
  {
    if(strcmp(argv[argz], "-correl") == 0)
    {
      do_correl = 1;
      sscanf(argv[++argz], "%d", &t_max);
    }
    else
    {
      fprintf(stderr, "unkown option: %s\n", argv[argz]);
      exit(1);
    }
    argz++;
  }

  if(argc-argz < 2)
  {
    fprintf(stderr, "USAGE: %s [OPTIONS] <t_min> <out-file>\n", argv[0]); 
    fprintf(stderr, "    OPTIONS:\n");
    fprintf(stderr, "    -correl <t>: correlation functions till <t>\n");
    exit(1);
  }

  t_min = atoi(argv[argz++]);
  stat = llstat_new_jackknife(-1,1,10,50);
  stat2 = llstat_new_jackknife(-1,1,10,50);
  stat_e = llstat_new(-3,1,10);
  while(argz<argc)
  {
    file = fopen(argv[argz++], "r");
    if(file == NULL)
    {
      fprintf(stderr, "Can't open file %s", argv[argz-1]);
      exit(2);
    }
    
    t_last = 0;                                   /* obtain file size */
    while(!feof(file))
    {
      fgets(line, MAXLINE, file); 
      if(feof(file))
	continue;
      if(line[0] == '#')
      {
	printf("%s", line);
	continue;
      }
      num_scanned = sscanf(line, "%d %lf %lf", &time, &energy, &mag, &mag2);
	if(time>t_last)
	  t_last = time;
    }
    fclose(file);
    max_data = t_last;
    TFNE_MALLOC_AH(value, max_data+1, double, "correl"); 
 

    file = fopen(argv[argz-1], "r");
    t_last = 0;                                           /* treat data */
    while(!feof(file))
    {
      fgets(line, MAXLINE, file);
      if(feof(file))
	continue;
      if(line[0] == '#')
      {
	printf("%s", line);
	continue;
      }
      num_scanned = sscanf(line, "%d %lf %lf %lf", &time, &energy, &mag, &mag2);
      if((num_scanned == 4)&&(time>t_min))
      {
	llstat_insert(stat, fabs(mag));
	llstat_insert(stat2, fabs(mag2));
	llstat_insert(stat_e, energy);
	if(time>t_last)
	  t_last = time;
	if(time < max_data)  /* store data point for correlation function */
	{
	  if(c_value == ENERGY)
	    value[time] = energy;
	  else if(c_value = MAG)
	    value[time] =fabs(mag);
	  else if(c_value = MAG2)
	    value[time] =fabs(mag2);
	}
      }
      /*printf("%s", line);*/
    }

    fclose(file);

    if(do_correl)
    {
      sprintf(filename, "%s.corr", argv[argz-1]);
      file = fopen(filename, "w");
      for(delta_time=0; delta_time<t_max; delta_time++)  
      {             /* calculate correlation function: Newman/Barkema p. 63 */
	sum1=0.0; sum2=0.0; sum3=0.0;
	for(time=1; time<=t_last-delta_time; time++)
        {
	  sum1 += value[time]*value[time+delta_time];
	  sum2 += value[time];
	  sum3 += value[time+delta_time];
	}
	correl = (sum1-sum2*sum3/(t_last-delta_time))/(t_last-delta_time);
	if(delta_time==0)
	  correl_norm = correl;
	fprintf(file, "%d %f\n", delta_time, correl/correl_norm);
      }
      fclose(file);
    }
    free(value);
  }

  binder = llstat_binder2(stat, &binder_err);
  binder2 = llstat_binder2(stat2, &binder_err2);
  printf("ENERGY(65) MAG(65) BINDER_M(65) BINDER_M2(65): " 
	 "%f %f %f %f %f %f %f %f\n", 
	 llstat_mean(stat_e), llstat_error(stat_e), 
	 llstat_mean(stat), llstat_error(stat), 
	 binder, binder_err, binder2, binder_err2);

  llstat_delete(stat);
  llstat_delete(stat2);
  llstat_delete(stat_e);
  return(0);
}
