/*** include-file for statistical analysis            ***/
/*** A.K. Hartmann January 1995                       ***/
/*** Version 0.3.1.0  27.10.1999                      ***/

/*** Vers. History                                    ***/
/*** V. 0.3.10 int llstat: *table: (int) -> (double)  ***/

#ifndef _LLSTAT_H_
#define _LLSTAT_H_

/** data types: **/

#define _LLSTAT_NOM_      9     /* No. of (statistical) moments */
#define _LLSTAT_MAGIC_ 1299
#define _LLCORR_MAGIC_ 1298

/* holds statistical informations for a set of numbers:   */
/* empirical distribution, No. of Numbers, sum of numbers */
/* and sum of squares.                                    */
typedef struct
{
  double           from, to;        /* range of table               */
  double              delta;        /* width of baskets             */  
  int                n_bask;        /* number of baskets            */
  double             *table;        /* empirical distribution       */
  double          low, high;        /* No. of data out of range     */
  double  sum[_LLSTAT_NOM_];        /* sum of 0, numbers, numbers^2 */
  int         num_jackknife;        /* number k of jacknife blocks  */
  int         act_jackknife;        /* currect jacknife block       */
  double    **sum_jackknife;        /* sum of every k'th value      */
  int                 magic;        /* for security                 */
} llstat_t;


/* holds empirical correlation functions = set of (x,y) numbers */
typedef struct
{
  double      from_x, to_x;       /* x-range of table               */
  double           delta_x;       /* x-width of baskets             */  
  int             n_bask_x;       /* number of x-baskets            */
  int                  num;       /* number of pairs in table       */
  llstat_t          **stat;       /* empirical ditribution          */
  llstat_t *low_x, *high_x;       /* values of data out of range    */
  int                magic;       /* for security                   */
} llcorr_t;


/** function prototypes: **/

extern llstat_t *llstat_new(double from, double to, int n_bask);
llstat_t *llstat_new_jackknife(double from, double to, int n_bask,
			       int num_jackknife);
extern void llstat_clear(llstat_t *stat);
extern void llstat_delete(llstat_t *stat);
extern void llstat_insert(llstat_t *stat, double number);
extern double llstat_dens(llstat_t *stat, double x);
extern void llstat_insert_mult(llstat_t *stat, double number, double times);
extern double llstat_no(llstat_t *stat);
extern double llstat_num(llstat_t *stat);
extern double llstat_mean(llstat_t *stat);
extern double llstat_mean_jackknife(llstat_t *stat, int id_jackknife);
extern double llstat_moment_jackknife(llstat_t *stat, int moment, 
				      int id_jackknife);
extern double llstat_median(llstat_t *stat);
extern double llstat_moment(llstat_t *stat, int n);
extern double llstat_error_moment(llstat_t *stat, int n);
extern double llstat_var(llstat_t *stat);
extern double llstat_varvar(llstat_t *stat);
extern double llstat_error(llstat_t *stat);
extern double llstat_errorvar(llstat_t *stat);
extern void llstat_write_log(llstat_t *stat, FILE *file, double factor);
extern void llstat_write(llstat_t *stat, FILE *file);
extern void llstat_print(llstat_t *stat);
extern void llstat_w_pc(llstat_t *stat, FILE *file, double density,int number);
extern void llstat_p_pc(llstat_t *stat, double density, int number);
extern void llstat_max(llstat_t * stat, double *max_x, double *max_y);
extern double llstat_int(llstat_t *stat, double from, double to);
extern double llstat_random_0_1();
extern double llstat_random_norm();
extern int llstat_random_0_n(int n);
extern void llstat_random_init();
extern llcorr_t *llcorr_new_jackknife(double from_x, double to_x, 
				      double from_y, double to_y,
				     int n_bask_x, int n_bask_y, int num_jn_y);
extern llcorr_t *llcorr_new(double from_x, double to_x, double from_y, 
			    double to_y, int n_bask_x, int n_bask_y);
extern void llcorr_clear(llcorr_t *corr);
extern void llcorr_delete(llcorr_t *corr);
extern void llcorr_insert(llcorr_t *corr, double x, double y);
extern double llcorr_mean(llcorr_t *corr, double x);
extern double llcorr_moment(llcorr_t *corr, int n);
extern void llcorr_write(llcorr_t *corr, FILE *file);
extern void llcorr_write_log(llcorr_t *corr, FILE *file, double factor);
extern void llcorr_print(llcorr_t *corr);
extern int llcorr_add_stat(llcorr_t *corr, llstat_t *stat);
extern double llstat_binder(llstat_t *m2_stat, llstat_t *m4_stat, 
							double *error65);
extern double llstat_binder2(llstat_t *m_stat, double *error65);

#endif            /* #ifndef _LLSTAT_H_ */



